import collections
import os
from typing import Dict, List

from large_scale_deploy.config_model.host import InventoryHostInfo
from large_scale_deploy.config_model.progress_json import HostStatus, ProgressInfo


class DisplayRow:

    def __init__(self, row_name):
        self.row_name = row_name
        self._status_map = {status: 0 for status in HostStatus.ALL_STATUS}
        self.progress = 0

    def update_status(self, status_name, count):
        self._status_map[status_name] = count

    def update_progress(self, progress):
        self.progress = str(float(progress) * 100) + "%"

    def get_row_field_list(self):
        return [self.row_name] + [self._status_map.get(status) for status in HostStatus.ALL_STATUS] + [self.progress]


class DisplayTable:
    _ROW_TITLES = ("Task\\Status",) + HostStatus.ALL_STATUS + ("progress",)
    _MAX_STEP_LEN = 32

    def __init__(self, row_names):
        self._rows = [DisplayRow(row_name) for row_name in row_names]
        len_list = [self._MAX_STEP_LEN] + [len(title) + 2 for title in self._ROW_TITLES[1:]]
        self._format_template = " | ".join(f"{{:<{length}}}" for length in len_list)
        self._row_map: Dict[str, DisplayRow] = {row.row_name: row for row in self._rows}

    def update_status(self, row_name, status_name, count):
        if row_name not in self._row_map:
            return
        self._row_map.get(row_name).update_status(status_name, count)

    def update_progress(self, row_name, progress):
        if row_name not in self._row_map:
            return
        self._row_map.get(row_name).update_progress(progress)

    def display(self):
        if not self._rows:
            return ""
        display_rows = [self._format_template.format(*self._ROW_TITLES)]
        for row in self._rows:
            display_rows.append(self._format_template.format(*row.get_row_field_list()))
        return os.linesep.join(display_rows)


class StatusCount:

    def __init__(self, status: str, count: int = 0):
        self.status = status
        self.count = count


class DeployStep:

    def __init__(self, step_name: str, status_count_list: List[StatusCount], progress: float):
        self.status_count_list = status_count_list
        self.step_name = step_name
        self.progress = progress


class ClusterDeployProgressViewer:
    _NON_HOST = ("localhost",)

    def __init__(self, deploy_nodes: List[InventoryHostInfo], all_worker: List[InventoryHostInfo]):
        self._deploy_nodes = [deploy_node.ip for deploy_node in deploy_nodes]
        self._all_worker = [worker.ip for worker in all_worker]
        self._deploy_steps = []
        self._deploy_node_step_info_map: Dict[str, Dict[str, DeployStep]] = {}
        for deploy_node in self._deploy_nodes:
            self._deploy_node_step_info_map[deploy_node] = {}
        self._display_table = DisplayTable([])

    def update(self, deploy_node_ip, progress_json: Dict):
        if not deploy_node_ip or not progress_json:
            return
        progress_info = ProgressInfo.from_json(progress_json)
        if not self._deploy_steps:
            self._deploy_steps = [playbook.desc_en for playbook in progress_info.playbooks]
            self._display_table = DisplayTable(self._deploy_steps)
        step_info_map = self._deploy_node_step_info_map.get(deploy_node_ip)
        for playbook in progress_info.playbooks:
            host_info_list = [host_info for host_info in playbook.host_info_list if host_info.ip not in self._NON_HOST]
            status_count_map = dict(collections.Counter([host_info.status for host_info in host_info_list]))
            status_count_list = []
            for status in HostStatus.ALL_STATUS:
                count = status_count_map.get(status, 0)
                status_count_list.append(StatusCount(status, count))
            step_info_map[playbook.desc_en] = DeployStep(playbook.desc_en, status_count_list, playbook.progress)

    def display(self):
        for step in self._deploy_steps:
            step_progress_sum = 0
            step_host_count_map: Dict[str, StatusCount] = {}
            for status in HostStatus.ALL_STATUS:
                step_host_count_map[status] = StatusCount(status)
            for deploy_node, step_info_map in self._deploy_node_step_info_map.items():
                step_info = step_info_map.get(step)
                if not step_info:
                    continue
                step_progress_sum += step_info.progress
                for status_count in step_info.status_count_list:
                    step_host_count_map.get(status_count.status).count += status_count.count
            for status_count in step_host_count_map.values():
                self._display_table.update_status(step, status_count.status, status_count.count)
            progress = round(step_progress_sum / len(self._deploy_node_step_info_map), 2)
            self._display_table.update_progress(step, progress)
        return self._display_table.display()
