import json
import os
import queue
import sys
import time
from concurrent.futures import ThreadPoolExecutor
from functools import partial

from jobs import PrepareJob
from large_scale_deploy.config_model.inventory import LargeScaleInventory
from large_scale_deploy.handler.inventory_spliter import InventorySpliter
from large_scale_deploy.process.remote_deploy_task import RemoteDeployTask, HostError
from large_scale_deploy.tools.decorators import process_output
from large_scale_deploy.tools.errors import ConfigrationError, LargeScaleDeployFailed
from large_scale_deploy.tools.network_tool import NetworkTool
from large_scale_deploy.tools.spread_tool import ConnHostInfo, SpreadTool, SpreadManager
from large_scale_deploy.view.viewer import ClusterDeployProgressViewer
from module_utils.path_manager import LargeScalePath, TmpPath, PathManager
from ascend_deployer import CLI


class LargeScaleDeployer:
    _MAX_DEPLOY_WAIT_TIME = 8 * 60 * 60
    _ROUND_WAIT_TIME = 30

    def __init__(self):
        self._large_scale_inventory = LargeScaleInventory.parse(LargeScalePath.INVENTORY_FILE_PATH)
        self._subgroups = self._generate_subgroups()
        self._prepare_job = PrepareJob()
        self._io_workers_num = len(self._subgroups)
        self._thread_pool = ThreadPoolExecutor(max_workers=self._io_workers_num)
        self._progress_table_viewer = ClusterDeployProgressViewer(self._large_scale_inventory.deploy_node,
                                                                  self._large_scale_inventory.worker)
        self._host_error_que = queue.Queue()

    def _generate_subgroups(self):
        spliter = InventorySpliter(self._large_scale_inventory)
        if self._large_scale_inventory.deploy_node:
            subgroups = spliter.split_by_deploy_node()
        else:
            subgroups = spliter.split_by_network()
        return subgroups

    def _get_src_host(self):
        ips = NetworkTool.get_local_host_ips()[1]
        local_ip_set = {ip.strip() for ip in ips}
        for worker in self._large_scale_inventory.worker:
            if worker.ip in local_ip_set:
                return ConnHostInfo.from_ansible_host_info(worker.to_info_dict())
        raise ConfigrationError(f"Execute host must in worker nodes. local ip(s) {list(local_ip_set)} not in worker.")

    def _spread_deployer_pkg(self):
        deploy_nodes = []
        for subgroup in self._subgroups:
            deploy_nodes.append(ConnHostInfo.from_ansible_host_info(subgroup.deploy_node.to_info_dict()))
        root_spread_node = SpreadTool.analyse_spread_tree(deploy_nodes, self._get_src_host())
        spread_res = SpreadManager(root_spread_node, True, deploy_nodes).start()
        if not spread_res.result:
            raise LargeScaleDeployFailed(os.linesep.join(spread_res.msg_list))

    def _start_deploy(self, ansible_args):
        futures = []
        for sub_group in self._subgroups:
            task = RemoteDeployTask(sub_group.deploy_node_conn_info, sub_group.inventory, ansible_args,
                                    self._host_error_que)
            futures.append(self._thread_pool.submit(task.start))
        return futures

    @staticmethod
    def _query_progress_json(host_res_dir):
        try:
            full_host_res_dir = os.path.join(LargeScalePath.REMOTE_HOST_RESULTS, host_res_dir)
            progress_json = os.path.join(full_host_res_dir, TmpPath.PROGRESS_JSON_NAME)
            if not os.path.exists(progress_json):
                return {}
            with open(progress_json, encoding="utf8") as f:
                progress_json = json.load(f)
        except Exception:
            progress_json = {}
        return progress_json

    def _build_host_task_error_info(self):
        error_msg_list = []
        while not self._host_error_que.empty():
            host_error: HostError = self._host_error_que.get()
            error_msg_list.append(f"Host: {host_error.host} task error: {host_error.error_info_list}")
        return os.linesep.join(error_msg_list)

    @staticmethod
    def finish_evnet(futures):
        return all(future.done() for future in futures)

    def _start_collect_progress(self, finish_event):
        start_time = time.time()
        while not finish_event():
            self._update_and_print()
            time.sleep(self._ROUND_WAIT_TIME)
            if time.time() - start_time > self._MAX_DEPLOY_WAIT_TIME:
                raise LargeScaleDeployFailed("Deploy time out.")
        self._update_and_print()

    def _update_and_print(self):
        for host_res_dir in os.listdir(LargeScalePath.REMOTE_HOST_RESULTS):
            progres_json = self._query_progress_json(host_res_dir)
            self._progress_table_viewer.update(host_res_dir, progres_json)
        display_str = self._progress_table_viewer.display()
        if display_str:
            print(display_str)

    def start(self, ansible_args):
        try:
            PathManager.init_large_scale_dirs()
            # self._spread_deployer_pkg()
            futures = self._start_deploy(ansible_args)
            finish_evnet = partial(self.finish_evnet, futures)
            self._start_collect_progress(finish_evnet)
            if not self._host_error_que.empty():
                raise LargeScaleDeployFailed(self._build_host_task_error_info())
        finally:
            self._thread_pool.shutdown()


class LargeScaleCli(CLI):

    def __init__(self):
        super().__init__("large-scale-deployer", "Manage Ascend Packages and dependence packages for specified OS "
                                                 "in large scale deploy.")
        self.parser.add_argument("--retry")
        self.args = ' '.join(sys.argv[1:])

    def run(self):
        self._check_args()
        self._license_agreement()
        LargeScaleDeployer().start(self.args)


@process_output()
def main():
    os.umask(0o022)
    LargeScaleCli().run()


if __name__ == '__main__':
    return_code, stdout, stderr = main()
    stdout and sys.stdout.write(stdout)
    stderr and sys.stdout.write(stderr)
    sys.exit(return_code)
