#!/usr/bin/env python3
# coding: utf-8
# Copyright 2024 Huawei Technologies Co., Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
# ===========================================================================
import glob
import os
import platform

from ansible.module_utils.check_output_manager import check_event
from ansible.module_utils.check_utils import CheckUtil as util


class FrameCheck:

    def __init__(self, module, npu_info, error_messages):
        self.tags = module.params.get("tags")
        self.python_version = module.params.get("python_version")
        self.packages = module.params.get("packages")
        self.error_messages = error_messages
        self.npu_info = npu_info
        self.resources_dir = os.path.join(module.params.get("ascend_deployer_work_dir"), "resources")

    def _get_tfadaptor_path(self):
        pylib_path = os.path.join(self.resources_dir, "pylibs", '.'.join(self.python_version.split(".")[:2]))
        arch = platform.machine()
        tensorflow_pkg = glob.glob(os.path.join(pylib_path, "tensorflow_cpu-*{}.whl".format(arch)))
        tensorflow_pkg += glob.glob(os.path.join(pylib_path, "tensorflow-*{}.whl".format(arch)))
        if not tensorflow_pkg:
            return ""
        tensorflow_pkg.sort(reverse=True)
        tensorflow_info = os.path.basename(tensorflow_pkg[0]).split('-')
        tensorflow_version = ""
        if len(tensorflow_info) > 1:
            tensorflow_version = tensorflow_info[1]
        tfadaptor_path = glob.glob(os.path.join(pylib_path, "npu_*{}*{}.whl".format(tensorflow_version, arch)))
        return tfadaptor_path

    @check_event
    def check_torch(self):
        scene = self.npu_info.get("scene", "")

        if scene == "a910b":
            self.check_kernels("910b")
        if scene == "a910_93":
            self.check_kernels("910_93")

        skip_tags = {"toolkit", "nnae", "pytorch_dev", "pytorch_run", "auto"}
        nnae_pkg = self.packages.get("nnae")
        toolkit_pkg = self.packages.get("toolkit")
        if skip_tags.intersection(set(self.tags)) and (nnae_pkg or toolkit_pkg):
            return

        toolkit_path = "/usr/local/Ascend/ascend-toolkit/set_env.sh"
        nnae_path = "/usr/local/Ascend/nnae/set_env.sh"
        if not os.path.exists(toolkit_path) and not os.path.exists(nnae_path):
            util.record_error("[ASCEND][ERROR] Please install toolkit or nnae before install pytorch.",
                              self.error_messages)

    def check_kernels(self, scene):
        # 1. Check whether kernels have been installed.
        toolkit_kernels_path = "/usr/local/Ascend/ascend-toolkit/latest/opp/built-in/op_impl/ai_core/tbe/kernel/ascend{}/".format(scene)
        nnae_kernels_path = "/usr/local/Ascend/nnae/latest/opp/built-in/op_impl/ai_core/tbe/kernel/ascend{}/".format(scene)
        if os.path.exists(toolkit_kernels_path) or os.path.exists(nnae_kernels_path):
            return
        # 2. Check whether the installed tags contain kernels, pytorch_dev, or pytorch_run.
        skip_tags = {"pytorch_dev", "pytorch_run", "kernels"}
        if skip_tags & set(self.tags):
            return
        # 3. Check whether the kernels package exists during installation in the auto scenario.
        kernels_pkg = self.packages.get("kernels")
        if "auto" in self.tags and kernels_pkg:
            return
        # 4. In other cases.
        util.record_error(
            "[ASCEND][ERROR] For Atlas A2 training series products, please install kernels before install pytorch.",
            self.error_messages)

    @check_event
    def check_tensorflow(self):
        if "3.10" in self.python_version:
            util.record_error("[ASCEND][ERROR] Tensorflow does not support python3.10.* and above. "
                              "Please use an earlier python version.", self.error_messages)

        skip_tags = {"tensorflow_dev", "tensorflow_run", "auto"}
        nnae_pkg = self.packages.get("nnae")
        toolkit_pkg = self.packages.get("toolkit")
        tfplugin_pkg = self.packages.get("tfplugin")
        tfadaptor_pkg = self._get_tfadaptor_path()
        """
        CANN 8.0.0之前，TensorFlow需要tfplugin+toolkit或tfplugin+nnae组合
        CANN 8.0.0及之后，提供tfadaptor的whl包代替tfplugin，whl包跟随tensorflow一起下载
        """
        need_pkg_existed = True if (nnae_pkg or toolkit_pkg) and (tfplugin_pkg or tfadaptor_pkg) else False
        if skip_tags.intersection(set(self.tags)) and need_pkg_existed:
            return

        toolkit_path = "/usr/local/Ascend/ascend-toolkit/set_env.sh"
        nnae_path = "/usr/local/Ascend/nnae/set_env.sh"
        tfplugin_path = "/usr/local/Ascend/tfplugin/set_env.sh"
        install_toolkit = ((set(self.tags).intersection(skip_tags.union({"toolkit"})) and toolkit_pkg)
                           or os.path.exists(toolkit_path))
        install_nnae = ((set(self.tags).intersection(skip_tags.union({"nnae"})) and nnae_pkg)
                        or os.path.exists(nnae_path))
        install_tfplugin = ((set(self.tags).intersection(skip_tags.union({"tfplugin"})) and tfplugin_pkg)
                            or os.path.exists(tfplugin_path))
        if not install_toolkit and not install_nnae:
            util.record_error("[ASCEND][ERROR] Please install toolkit or nnae before install tensorflow.",
                              self.error_messages)
        elif not install_tfplugin and not tfadaptor_pkg:
            util.record_error(
                "[ASCEND][ERROR] Please install tfplugin or download tfadaptor before install tensorflow.",
                self.error_messages)

    @check_event
    def check_mindspore(self):
        if "3.12" in self.python_version:
            util.record_error("[ASCEND][ERROR] Mindspore does not support python3.12.* and above. "
                              "Please use an earlier python version.", self.error_messages)

        skip_tags = {"toolkit", "nnae", "auto"}
        nnae_pkg = self.packages.get("nnae")
        toolkit_pkg = self.packages.get("toolkit")
        if skip_tags.intersection(set(self.tags)) and (nnae_pkg or toolkit_pkg):
            return

        toolkit_path = "/usr/local/Ascend/ascend-toolkit/set_env.sh"
        nnae_path = "/usr/local/Ascend/nnae/set_env.sh"
        if not os.path.exists(toolkit_path) and not os.path.exists(nnae_path):
            util.record_error("[ASCEND][ERROR] Please install toolkit or nnae before install mindspore.",
                              self.error_messages)

